//
// Created by huli on 2023/8/8.
//

#include "dispatch_led.h"
#include "../system/system_communication.h"
#include "../dispatch/dispatch_manager.h"


Dispatch_led::Dispatch_led()
{
	m_dispatch_led_status = DISPATCH_LED_UNKNOW;
	m_dispatch_id = 0;
	m_led_id = 0;

}

Dispatch_led::~Dispatch_led()
{
	dispatch_led_uninit();
}


//调度单片机 初始化,  单片机id, 0或者1, 单片机方向, 1是入口, 2是出口
Error_manager Dispatch_led::dispatch_led_init(int dispatch_id, int led_id, int led_direction)
{
	m_dispatch_id =	dispatch_id;
	m_led_id = led_id;
	m_led_direction = led_direction;
	mp_updata_sql_thread = nullptr ;
	mp_updata_led_thread = nullptr ;
	m_led_updata_time = std::chrono::system_clock::now();

	m_led_protocol.led_protocol_init();

	//	线程默认开启
	m_updata_sql_condition.reset(false, true, false);
	mp_updata_sql_thread = new std::thread(&Dispatch_led::updata_sql_thread_fun, this);

	//	线程默认开启
	m_updata_led_condition.reset(false, true, false);
	mp_updata_led_thread = new std::thread(&Dispatch_led::updata_led_thread_fun, this);

	m_dispatch_led_status = DISPATCH_LED_READY;


	return Error_code::SUCCESS;
}
//调度单片机 反初始化
Error_manager Dispatch_led::dispatch_led_uninit()
{
	if (mp_updata_sql_thread)
	{
		m_updata_sql_condition.kill_all();
	}
	if (mp_updata_sql_thread)
	{
		mp_updata_sql_thread->join();
		delete mp_updata_sql_thread;
		mp_updata_sql_thread = NULL;
	}

	if (mp_updata_led_thread)
	{
		m_updata_led_condition.kill_all();
	}
	if (mp_updata_led_thread)
	{
		mp_updata_led_thread->join();
		delete mp_updata_led_thread;
		mp_updata_led_thread = NULL;
	}

	m_dispatch_led_status = Dispatch_led::DISPATCH_LED_UNKNOW;

	return Error_code::SUCCESS;
}

//调度LED 执行状态消息, LED通过tcp发过来的原始字符串,
Error_manager Dispatch_led::execute_for_led_data_msg_new(std::string led_data)
{
	return Error_code::SUCCESS;
}

Dispatch_led::Dispatch_led_status Dispatch_led::get_dispatch_led_status()
{
	return m_dispatch_led_status;
}

//任务执行线程, 负责对上面的排队指令表
void Dispatch_led::updata_sql_thread_fun()
{
	LOG(INFO) << " Dispatch_led::updata_sql_thread_fun() start " << this;
	Error_manager t_error;

	while (m_updata_sql_condition.is_alive())
	{
		m_updata_sql_condition.wait();
		if (m_updata_sql_condition.is_alive())
		{
			std::this_thread::sleep_for(std::chrono::milliseconds(1));
			std::this_thread::sleep_for(std::chrono::milliseconds(100));


			//更新数据库的缓存。
			t_error = updata_sql_data();
			std::this_thread::yield();
		}
	}
	LOG(INFO) << " Dispatch_led::updata_sql_thread_fun() end " << this;

}
//更新数据库的缓存。
Error_manager Dispatch_led::updata_sql_data()
{
	std::unique_lock<std::mutex> t_lock2(m_lock);
	std::unique_lock<std::mutex> t_lock(Dispatch_manager::get_instance_references().m_dispatch_command.m_lock);
	Dispatch_command::Dispatch_command_collect & t_dispatch_command_collect_now = Dispatch_manager::get_instance_references().m_dispatch_command.m_dispatch_command_collect_now;
	Dispatch_communication::Dispatch_plc_status_from_plc_to_manager & t_dispatch_plc_status_from_plc_to_manager = Dispatch_manager::get_instance_references().m_dispatch_plc.m_dispatch_plc_status_from_plc_to_manager;

	int t_led_carrier_status = 0;
	if ( (Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_WORKING && Dispatch_manager::get_instance_references().m_dispatch_process_type == Common_data::DISPATCH_PROCESS_STORE)
		 || (Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_REALLOCATE && Dispatch_manager::get_instance_references().m_dispatch_process_type == Common_data::DISPATCH_PROCESS_REALLOCATE)
	)
	{
		t_led_carrier_status = 1;
	}
	else if ( (Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_WORKING && Dispatch_manager::get_instance_references().m_dispatch_process_type == Common_data::DISPATCH_PROCESS_PICKUP)
			  || (Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_REALLOCATE && Dispatch_manager::get_instance_references().m_dispatch_process_type == Common_data::DISPATCH_PROCESS_REVOCATION)
			  || (Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_REALLOCATE && Dispatch_manager::get_instance_references().m_dispatch_process_type == Common_data::DISPATCH_PROCESS_REVOCATION_EX)
	)
	{
		t_led_carrier_status = 2;
	}
	else if ( Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_READY
			  || Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_REQUEST
			  || Dispatch_manager::get_instance_references().m_dispatch_manager_status == Dispatch_manager::E_DISPATCH_MANAGER_RESPONSE )
	{
		t_led_carrier_status = 3;
	}
	else
	{
		t_led_carrier_status = 4;
	}

//	t_led_carrier_status = 1;

	//检查并更新led缓存, plc机器人状态
	if ( m_led_carrier_status != t_led_carrier_status )
	{
		//更新led显示之前，刷新一次数据库
		Dispatch_manager::get_instance_references().m_dispatch_command.query_all_dispatch_command_ex_unlock();

		m_led_carrier_status = t_led_carrier_status;
		m_led_update_flag = true;
		//检查并更新led缓存, 指令信息
		if ( m_led_carrier_status == 1 )
		{
			m_command_passway_id=0;
			m_command_space_lable_id=0;
			m_command_car_number.clear();
			LOG(INFO) << " asdasdasdasdasdasdasdaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaasdasdasda t_dispatch_command_collect_now.m_dispatch_command_map_park_1.size() = " << t_dispatch_command_collect_now.m_dispatch_command_map_park_1.size();
			for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_park_1.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_park_1.end(); ++iter)
			{
				m_command_passway_id = iter->second.m_import_id;
				m_command_space_lable_id = iter->second.m_space_info_msg.table_id();
				m_command_car_number = iter->second.m_car_number;
				LOG(INFO) << " m_command_passway_id = " << m_command_passway_id;
				LOG(INFO) << " m_command_space_lable_id = " << m_command_space_lable_id;
				LOG(INFO) << " m_command_car_number = " << m_command_car_number;
			}

		}
		else if ( m_led_carrier_status == 2 )
		{
			m_command_passway_id=0;
			m_command_space_lable_id=0;
			m_command_car_number.clear();
			LOG(INFO) << " asdasdasdasdasdasdasdaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaasdasdasda t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.size() = " << t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.size();
			for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.end(); ++iter)
			{
				m_command_passway_id = iter->second.m_export_id;
				m_command_space_lable_id = iter->second.m_space_info_msg.table_id();
				m_command_car_number = iter->second.m_car_number;
				LOG(INFO) << " m_command_passway_id = " << m_command_passway_id;
				LOG(INFO) << " m_command_space_lable_id = " << m_command_space_lable_id;
				LOG(INFO) << " m_command_car_number = " << m_command_car_number;
			}
		}
		else if ( m_led_carrier_status == 3 )
		{
			m_command_passway_id=0;
			m_command_space_lable_id=0;
			m_command_car_number.clear();
			LOG(INFO) << " qweeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee  m_led_carrier_status == 3 = " << 3;
		}
		else
		{
			m_command_passway_id=0;
			m_command_space_lable_id=0;
			m_command_car_number.clear();
			LOG(INFO) << " dddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd  m_led_carrier_status == 0 = " << 0;

		}
	}

	//检查并更新led缓存, 出入口状态
	int t_plc_passway_status = ( ( t_dispatch_plc_status_from_plc_to_manager.m_plc_passway_status >> ((m_led_id-1)*2) ) & 0x03);
	if ( m_led_passway_status != t_plc_passway_status )
	{
		m_led_passway_status = t_plc_passway_status;
		m_led_update_flag = true;
	}
	int t_passway_has_car = ( (t_dispatch_plc_status_from_plc_to_manager.m_passway_vector[m_led_id-1].m_sensor_2 >> 2)  & 0x01);
	if ( m_passway_has_car != t_passway_has_car )
	{
		m_passway_has_car = t_passway_has_car;
		m_led_update_flag = true;
	}

	//检查并更新led缓存 //通道口车牌号， 只有停在出入口才会显示
	std::string t_passway_car_number_inlet;
	if ( m_led_id == 1 || m_led_id == 2 )
	{
		for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_park_0.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_park_0.end(); ++iter)
		{
			if ( iter->second.m_import_id == m_led_id )
			{
				t_passway_car_number_inlet = iter->second.m_car_number;
			}
		}
	}
	if ( m_passway_car_number_inlet != t_passway_car_number_inlet )
	{
		m_passway_car_number_inlet = t_passway_car_number_inlet;
		m_led_update_flag = true;
	}

	std::string t_passway_car_number_outlet;
	if ( m_led_id == 1 || m_led_id == 1 )
	{
		for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_pick_2.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_pick_2.end(); ++iter)
		{
			if ( iter->second.m_export_id == m_led_id )
			{
				t_passway_car_number_outlet = iter->second.m_car_number;
			}
		}
		for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_park_2.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_park_2.end(); ++iter)
		{
			if ( iter->second.m_export_id == m_led_id )
			{
				t_passway_car_number_outlet = iter->second.m_car_number;
			}
		}
	}
	if ( m_passway_car_number_outlet != t_passway_car_number_outlet )
	{
		m_passway_car_number_outlet = t_passway_car_number_outlet;
		m_led_update_flag = true;
	}

	//检查并更新led缓存, //空闲小车中车位数量，
	if ( m_ready_small_middle_space_count != t_dispatch_command_collect_now.m_ready_small_space_count + t_dispatch_command_collect_now.m_ready_middle_space_count )
	{
		m_ready_small_middle_space_count = t_dispatch_command_collect_now.m_ready_small_space_count + t_dispatch_command_collect_now.m_ready_middle_space_count;
		m_led_update_flag = true;
	}

	//检查并更新led缓存, //空闲大车位数量，
	if ( m_ready_big_space_count != t_dispatch_command_collect_now.m_ready_big_space_count  )
	{
		m_ready_big_space_count = t_dispatch_command_collect_now.m_ready_big_space_count ;
		m_led_update_flag = true;
	}

	//检查并更新led缓存, //等候取车的信息 count， 不包括取车中
	if ( m_wait_pick_count != t_dispatch_command_collect_now.m_wait_total_pick_count )
	{
		m_wait_pick_count = t_dispatch_command_collect_now.m_wait_total_pick_count;
		m_led_update_flag = true;
	}
	//检查并更新led缓存, //等候取车的信息 字符串, 包括取车中
	std::string t_wait_pick_string = "候车:";
	int t_wait_pick_count=1;
	for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_map_pick_1.end(); ++iter)
	{
		char buf[1024]={0};
		sprintf(buf, "%d_%s ", t_wait_pick_count, iter->second.m_car_number.c_str());
		t_wait_pick_string += buf;
		t_wait_pick_count++;
	}
	for (auto iter = t_dispatch_command_collect_now.m_dispatch_command_vector_pick_0.begin(); iter != t_dispatch_command_collect_now.m_dispatch_command_vector_pick_0.end(); ++iter)
	{
		char buf[1024]={0};
		sprintf(buf, "%d_%s ", t_wait_pick_count, iter->m_car_number.c_str());
		t_wait_pick_string += buf;
		t_wait_pick_count++;
	}
	if ( m_wait_pick_string != t_wait_pick_string  )
	{
		m_wait_pick_string = t_wait_pick_string;
		m_led_update_flag2 = true;
	}


	return Error_code::SUCCESS;
}

//任务执行线程, 负责对下面的LED
void Dispatch_led::updata_led_thread_fun()
{
	LOG(INFO) << " Dispatch_led::updata_led_thread_fun() start " << this;
	Error_manager t_error;

	while (m_updata_led_condition.is_alive())
	{
		m_updata_led_condition.wait();
		if (m_updata_led_condition.is_alive())
		{
			std::this_thread::sleep_for(std::chrono::milliseconds(1));
			std::this_thread::sleep_for(std::chrono::seconds(1));

			//更新数据库的缓存。
			updata_led_data();
			std::this_thread::yield();


		}

	}
	LOG(INFO) << " Dispatch_led::updata_led_thread_fun() end " << this;

}
//更新LED的显示内容
Error_manager Dispatch_led::updata_led_data()
{
	std::unique_lock<std::mutex> t_lock2(m_lock);

	if ( m_led_id == 1 || m_led_id == 2 )
	{
//		std::cout << " huli test :::: " << " ************************************************************************************************************ = " << m_led_id << std::endl;

		if ( m_led_update_flag == true ||
			 m_led_update_flag2 == true ||
			 std::chrono::system_clock::now() - m_led_updata_time > std::chrono::seconds(10) )
		{
			//入口 1和2行
			if ( true )
			{
				m_led_update_flag = false;
				m_led_updata_time = std::chrono::system_clock::now();

				char buf1[1024] = {0};
				buf1[0] = 'A' + (m_dispatch_id / 10 - 1);
				buf1[1] = '1' + (m_dispatch_id % 10 - 1);
				buf1[2] = ' ';

				char buf2[1024];
				if (m_led_carrier_status == 4 || m_led_carrier_status == 0)
				{
					sprintf(buf2, "机器人维护中");
				}
				else if (m_led_carrier_status == 5)
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf2, "请等待");
					}
					else
					{
						sprintf(buf2, "机器人维护中");
					}
				}
				else if (m_led_carrier_status == 6)
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf2, "请前往其他车库");
					}
					else
					{
						sprintf(buf2, "机器人维护中");
					}
				}
				else if (m_led_carrier_status == 7)
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf2, "月度保养");
					}
					else
					{
						sprintf(buf2, "机器人维护保养");
					}
				}
				else if (!m_passway_car_number_inlet.empty())
				{
					sprintf(buf2, "%s 等候存车", m_passway_car_number_inlet.c_str());
				}
				else if (m_led_carrier_status == 1)
				{
					sprintf(buf2, "%s 存%d#->%02d", m_command_car_number.c_str(), m_command_passway_id,
							m_command_space_lable_id);
				}
				else if (m_led_carrier_status == 2)
				{
					sprintf(buf2, "%s 取%02d->%d#", m_command_car_number.c_str(), m_command_space_lable_id,
							m_command_passway_id);
				}
				else if (m_led_carrier_status == 3)
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf2, "空闲中");
					}
					else
					{
						sprintf(buf2, "机器人空闲");
					}
				}

				char buf3[1024];
				if (m_led_passway_status == 2 || m_led_passway_status == 3)
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf3, "%d#          候车数%02d", m_led_id, m_wait_pick_count);
					}
					else
					{
						sprintf(buf3, "%d#入口维护中  候车数%02d", m_led_id, m_wait_pick_count);
					}
				}
				else if (m_ready_small_middle_space_count == 0 && m_ready_big_space_count == 0)
				{
					sprintf(buf3, "%d#满库不可进  候车数%02d", m_led_id, m_wait_pick_count);
				}
				else if (m_led_passway_status == 0)
				{
					sprintf(buf3, "%d#繁忙不可进  候车数%02d", m_led_id, m_wait_pick_count);
				}
				else if (m_led_passway_status == 1 && m_passway_has_car == 1)
				{
					sprintf(buf3, "%d#可进请排队  候车数%02d", m_led_id, m_wait_pick_count);
				}
				else if (m_led_passway_status == 1 && m_passway_has_car == 0)
				{
					sprintf(buf3, "%d#空闲可进入  候车数%02d", m_led_id, m_wait_pick_count);
				}
				else
				{
					if (m_dispatch_id == 31 || m_dispatch_id == 32)
					{
						sprintf(buf3, "%d#          候车数%02d", m_led_id, m_wait_pick_count);
					}
					else
					{
						sprintf(buf3, "%d#入口维护中  候车数%02d", m_led_id, m_wait_pick_count);
					}
				}

				char buf4[1024];
				if (m_dispatch_id == 31 || m_dispatch_id == 32)
				{
					sprintf(buf4, "剩高车%02d\\n剩大车%02d", m_ready_big_space_count, m_ready_small_middle_space_count);

				}
				else
				{
					sprintf(buf4, "剩大车%02d\\n剩小车%02d", m_ready_big_space_count, m_ready_small_middle_space_count);
//				sprintf(buf4, "剩SUV %02d\\n剩轿车%02d", m_ready_big_space_count , m_ready_small_middle_space_count);
				}



//			std::cout << " huli test :::: " << " buf1 = " << buf1 << std::endl;
//			std::cout << " huli test :::: " << " buf2 = " << buf2 << std::endl;
//			std::cout << " huli test :::: " << " buf3 = " << buf3 << std::endl;
//			std::cout << " huli test :::: " << " buf4 = " << buf4 << std::endl;

				std::string tt;
				tt += buf1;
				tt += buf2;
				m_led_protocol.create_led_message(0, String_convert::utf8_to_gbk(tt), 0, 0, 192 * 2, 16);
				m_led_protocol.create_led_message(1, String_convert::utf8_to_gbk(buf3), 0, 16, 192 * 2, 16);
				m_led_protocol.create_led_message(2, String_convert::utf8_to_gbk(buf4), 192 * 2, 0, 64 * 2, 32);
				m_led_protocol.set_line_mode(2, 2, 1);
				Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(),
																			m_led_id);

				LOG(INFO) << " led iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii = " << m_led_id;
				LOG(INFO) << " m_led_carrier_status = " << m_led_carrier_status;
				LOG(INFO) << "  String_convert::utf8_to_gbk(tt) = " << tt;
				LOG(INFO) << " String_convert::utf8_to_gbk(buf3) = " << buf3;
				LOG(INFO) << " String_convert::utf8_to_gbk(buf4) = " << buf4;
			}

			//出口 3和4行
			if ( true )
			{
				m_led_update_flag = false;

				char buf1[1024] = {0};
				buf1[0] = 'A' + (m_dispatch_id/10-1);
				buf1[1] = '1' + (m_dispatch_id%10-1);
				buf1[2] = ' ';

				char buf2[1024];
				if ( m_led_carrier_status == 4 || m_led_carrier_status == 0)
				{
					sprintf(buf2, "机器人故障维护中");
				}
				else if ( m_led_carrier_status == 5 )
				{
					sprintf(buf2, "请等待");
				}
				else if ( m_led_carrier_status == 6 )
				{
					sprintf(buf2, "维护中，请联系现场管理人员");
				}
				else if ( m_led_carrier_status == 7 )
				{
					sprintf(buf2, "月度保养，请联系现场管理人员");
				}
				else if ( m_led_passway_status == 2 || m_led_passway_status == 3 )
				{
					//sprintf(buf2, "%d#维护中 候车数%02d", m_led_id, m_wait_pick_count);
					sprintf(buf2, "%d#        候车数%02d", m_led_id, m_wait_pick_count);
				}
				else if ( !m_passway_car_number_outlet.empty() )
				{
					sprintf(buf2, "%s 取车完成", m_passway_car_number_outlet.c_str());
				}
				else if ( m_led_carrier_status == 1 )
				{
					sprintf(buf2, "%s 存%d#->%02d", m_command_car_number.c_str(), m_command_passway_id, m_command_space_lable_id);
				}
				else if ( m_led_carrier_status == 2 )
				{
					sprintf(buf2, "%s 取%02d->%d#", m_command_car_number.c_str(), m_command_space_lable_id, m_command_passway_id);
				}
				else if ( m_led_carrier_status == 3 )
				{
					sprintf(buf2, "机器人空闲");
//					sprintf(buf2, "空闲中");
				}

//				sprintf(buf2, "取车中，请稍等！");

//				std::cout << " huli test :::: " << " buf1 = " << buf1 << std::endl;
//				std::cout << " huli test :::: " << " buf2 = " << buf2 << std::endl;

				std::string tt ;
				tt+=buf1;
				tt+=buf2;

				if ( m_dispatch_id == 31 && m_led_id == 3 )
				{
					std::string t_led3 = "\\FE001\\FO001";
					tt = t_led3 + tt;
					m_led_protocol.create_led_message(0, String_convert::utf8_to_gbk(tt), 0, 0, 256*2, 16*2);
					Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(0), m_led_id);
				}
				else
				{
					m_led_protocol.create_led_message(3, String_convert::utf8_to_gbk(tt), 0, 32, 256*2, 16);
					Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(0), m_led_id);
				}
				LOG(INFO) << " led oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo = " << m_led_id;
				LOG(INFO) << " String_convert::utf8_to_gbk(tt) = " << tt;

			}

			if ( m_led_update_flag2 == true )
			{
				m_led_update_flag2 = false;

				char buf3[1024];
				sprintf(buf3, "%s", m_wait_pick_string.c_str());

//				std::cout << " huli test :::: " << " buf3 = " << buf3 << std::endl;

				if ( m_dispatch_id == 31 && m_led_id == 3 )
				{
					std::string t_led3 = "\\FE001\\FO001";
					t_led3 = t_led3 + buf3;
					m_led_protocol.create_led_message(1, String_convert::utf8_to_gbk(t_led3), 0, 16*2, 256*2, 16*2);
					m_led_protocol.set_display_mode(1, 3);
					m_led_protocol.set_speed(1, 0, 1);
					Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(1), m_led_id);
				}
				else if ( m_dispatch_id == 31 && m_led_id == 4 )
				{
					m_led_protocol.create_led_message(1, String_convert::utf8_to_gbk(buf3), 0, 16, 256, 16);
					m_led_protocol.set_display_mode(1, 3);
					m_led_protocol.set_speed(1, 6, 1);
					Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(1), m_led_id);
				}
				else
				{
					m_led_protocol.create_led_message(4, String_convert::utf8_to_gbk(buf3), 0, 48, 256*2, 16);
//					m_led_protocol.set_display_mode(4, 3);
//					m_led_protocol.set_speed(4, 3, 1);
					Dispatch_network::get_instance_references().encapsulate_msg(m_led_protocol.get_led_msg_total_string(1), m_led_id);
				}
				LOG(INFO) << " led ppppppppppppppppppppppppppppppppppppppppppppppppppppppp = " << m_led_id;
				LOG(INFO) << " String_convert::utf8_to_gbk(buf3) = " << buf3;

			}
		}
	}

//	std::cout << " huli test :::: " << " ------------------------------------------------------------------------------------------------------------ = " << m_led_id << std::endl;

	return Error_code::SUCCESS;
}



















