//
// Created by huli on 2023/8/8.
//

#ifndef NNXX_TESTS_DISPATCH_LED_H
#define NNXX_TESTS_DISPATCH_LED_H


#include "../error_code/error_code.h"
#include <glog/logging.h>
#include <thread>
#include <mutex>
#include "../tool/thread_condition.h"
#include "../tool/common_data.h"
#include "../tool/time_tool.h"
#include "../task/task_base.h"
//#include "../message/dispatch_message.pb.h"
//#include "../message/led_msg.pb.h"
#include "../message/message.pb.h"
#include "../dispatch/dispatch_communication.h"
#include <google/protobuf/message.h>
#include <google/protobuf/util/json_util.h>
#include <google/protobuf/text_format.h>

#include "../led/led_protocol.h"
#include "../dispatch/dispatch_command.h"




//调度led通信
class Dispatch_led
{
public:
	//调度led状态
	enum Dispatch_led_status
	{
		DISPATCH_LED_UNKNOW               	= 0,    //未知
		DISPATCH_LED_READY               	= 1,    //准备，待机

		DISPATCH_LED_FAULT					= 10,	//故障
		DISPATCH_LED_DISCONNECT			= 11, 	//断连

	};
public:
	Dispatch_led();
	Dispatch_led(const Dispatch_led& other)= default;
	Dispatch_led& operator =(const Dispatch_led& other)= default;
	~Dispatch_led();
public://API functions
	//调度LED 初始化,  LEDid, 1~4, LED方向, 1是入口, 2是出口
	Error_manager dispatch_led_init(int dispatch_id, int led_id, int led_direction);
	//调度LED 反初始化
	Error_manager dispatch_led_uninit();

	//调度LED 执行状态消息, LED通过tcp发过来的原始字符串,
	Error_manager execute_for_led_data_msg_new(std::string led_data);

public://get or set member variable
	Dispatch_led_status get_dispatch_led_status();

protected://member functions
	//任务执行线程, 负责对上面的排队指令表
	void updata_sql_thread_fun();
	//更新数据库的缓存。
	Error_manager updata_sql_data();

	//任务执行线程, 负责对下面的LED
	void updata_led_thread_fun();
	//更新LED的显示内容
	Error_manager updata_led_data();
protected://member variable
	Dispatch_led_status							m_dispatch_led_status;//调度LED的状态


	int 										m_dispatch_id;					//设备id, 索引, 就是楚天车库的单元号.
	int 										m_led_id;						//LEDid, 1~4
	int 										m_led_direction;				//LED方向, 1是入口, 2是出口
	std::mutex									m_lock;	//锁

	//任务执行线程, 负责对上面的排队指令表
	std::thread*        						mp_updata_sql_thread;   			//执行的线程指针，内存由本类管理
	Thread_condition							m_updata_sql_condition;			//执行的条件变量
	//任务执行线程, 负责对下面的LED
	std::thread*        						mp_updata_led_thread;   			//数据更新的线程指针，内存由本类管理
	Thread_condition							m_updata_led_condition;				//数据更新的条件变量

	Dispatch_command::Dispatch_command_collect		m_dispatch_command_collect_last;		//排队指令信息的集合, last


	//led入口显示数据
	unsigned short			m_led_carrier_status= 0;			//搬运器状态 0=故障 1=存车 2=取车 3=空闲 4=维护
	int 					m_command_passway_id=0;				//指令通道口id
	int 					m_command_space_lable_id=0;			//指令车位标签id
	std::string				m_command_car_number;				//指令通道口车牌号

	int 					m_led_passway_status=0;				//出入口状态 0=不可进车, 1=可进车, 2=维护不可进车, 3=维护可进车
	int 					m_passway_has_car=0;				//出入口是否有车
	std::string				m_passway_car_number_inlet;				//通道口车牌号， 只有停在出入口才会显示
	std::string				m_passway_car_number_outlet;				//通道口车牌号， 只有停在出入口才会显示

	unsigned short			m_ready_small_middle_space_count = 0;			//空闲小车中车位数量，
	unsigned short			m_ready_big_space_count = 0;					//空闲大车位数量，

	int						m_wait_pick_count=0;					//等候取车的信息 count， 不包括取车中
	std::string				m_wait_pick_string;						//等候取车的信息 字符串, 包括取车中
	int						m_error_pick_count=0;					//故障取车的信息 count， 不包括取车中
	std::string				m_error_pick_string;					//故障取车的信息 字符串, 包括取车中

	bool 					m_led_update_flag = true;				//led 刷新标志位
	bool 					m_led_update_flag2 = true;				//led 刷新标志位
	std::chrono::system_clock::time_point		m_led_updata_time;	//led 更新时间

	//Led通信协议
	Led_protocol			m_led_protocol;							//Led通信协议

private:

};
    



#endif //NNXX_TESTS_DISPATCH_LED_H
