import sys

sys.path.append("..")
import tool.db_helper.mysqlhelper as mdb


# 数据库操作
# 可自行添加操作函数

class DBOperation:
    def __init__(self, db_ip, db_port, db_name, db_user, db_password):
        self._db = mdb.MySqLHelper(db_ip, db_port, db_name, db_user, db_password)

    def query_command_in_unit(self, unit):
        sql = "select * from command_queue WHERE unit=%s"
        return self._db.selectall(sql, unit)

    def query_pick_command_in_unit(self, unit):
        sql = "select * from command_queue where unit=%s and type=2 order by queue_id ASC"
        return self._db.selectall(sql, unit)

    def query_pick_command_in_unit_and_statu(self, unit, statu):
        sql = "select * from command_queue where unit=%s and statu=%s and type=2 order by queue_id ASC"
        return self._db.selectall(sql, (unit, statu))

    def query_command_all_in_unit_and_sort(self, unit):
        sql = "select * from command_queue where unit=%s order by type,statu DESC,queue_id DESC"
        return self._db.selectall(sql, unit)

    def query_command_all_in_unit_type_and_sort(self, unit, type):
        sql = "select * from command_queue where unit=%s and type=%s order by type,statu DESC,queue_id DESC"
        return self._db.selectall(sql, (unit, type))

    def query_command_all(self):
        sql = "select * from command_queue"
        return self._db.selectall(sql)

    def query_command_in_car_number(self, car_number):
        sql = "select * from command_queue WHERE car_number=%s"
        return self._db.selectall(sql, car_number)

    def query_command_in_primary_key(self, primary_key):
        sql = "select * from command_queue WHERE primary_key=%s"
        return self._db.selectall(sql, primary_key)

    def query_command_in_height_unit_and_statu(self, height, unit):
        sql = "select * from command_queue WHERE height>=%s and unit=%s and statu=0 and type=1"  # type=1 存车
        return self._db.selectall(sql, (height, unit))

    def query_space_in_car_number(self, car_number):
        sql = "select * from space WHERE car_number=%s"
        return self._db.selectall(sql, car_number)

    def query_space_in_primary_key(self, primary_key):
        sql = "select * from space WHERE primary_key=%s"
        return self._db.selectall(sql, primary_key)

    def query_space_in_unit_and_table_id(self, unit, table_id):
        sql = "select * from space WHERE unit=%s and table_id=%s"
        return self._db.selectall(sql, (unit, table_id))

    def query_space_in_height(self, height):
        sql = "select * from space WHERE height>=%s"
        return self._db.selectall(sql, height)

    def query_space_in_height_and_empty(self, height):
        sql = "select * from space WHERE height>=%s and statu=0 and car_number IS NULL"
        return self._db.selectall(sql, height)

    def query_space_in_height_unit_and_empty(self, height, unit):
        sql = "select * from space WHERE height>=%s and unit=%s and statu=0 and car_number IS NULL"
        return self._db.selectall(sql, (height, unit))

    def query_space_in_unit(self, unit):
        sql = "select * from space WHERE unit=%s"
        return self._db.selectall(sql, unit)

    def query_space_all(self):
        sql = "select * from space"
        return self._db.selectall(sql)

    def update_space_status(self, space_id, statu):
        sql = "update space set statu=%s where id=%s"
        return self._db.update(sql, (statu, space_id))

    def clear_space_data(self, space_id):
        sql = "update space set car_number=NULL,primary_key=NULL,statu=0,space_info=NULL,measure_info=NULL,plate_color=NULL,plate_type=NULL,plate_confidence=NULL,recognition_time=NULL,plate_full_image=NULL,plate_clip_image=NULL where id=%s"
        return self._db.update(sql, space_id)

    def query_vehicle_primary_key(self, car_number):
        sql = "select primary_key from vehicle where car_number=%s"
        return self._db.selectall(sql, car_number)

    def delete_command(self, car_number):
        sql = "delete from command_queue where car_number=%s"
        return self._db.delete(sql, car_number)

    # 查询一段时间内停车次数
    # period_type = 1 :当天内
    #               2 :当月内
    #               3 :自定义起止时间
    # date = "2023-03-28 15:22:07"
    def query_parkingTime_inPeriod(self, period_type, date):
        sql = "select count(in_time_start) from record where in_time_start like %s"
        if period_type == "day":
            date = date[0:10]
        elif period_type == "month":
            date = date[0:7]
        date = date + "%"  # 模糊查询

        result = self._db.selectall(sql, date)  # [{'count(in_time_start)': 116}]
        return result[0]["count(in_time_start)"]

    # 查询一段时间内停车车数
    # period_type = 1 :当天内
    #               2 :当月内
    #               3 :自定义起止时间
    def query_carQuantity_inPeriod(self, period_type, date):
        sql = "select count(distinct(car_number)) from record where in_time_start like %s"
        if period_type == "day":
            date = date[0:10]
        elif period_type == "month":
            date = date[0:7]
        date = date + "%"  # 模糊查询

        result = self._db.selectall(sql, date)  # [{'count(distinct(car_number))': 4}]
        return result[0]["count(distinct(car_number))"]
