#!/usr/bin/env python
# -*- coding:utf-8 -*-
# author:SunXiuWen
# make_time:2019/1/13
import sys
sys.path.append("..")
import pymysql
# 无法加载DButils包
from DBUtils.PooledDB import PooledDB
# from dbutils.pooled_db import PooledDB

"""
@功能：创建数据库连接池
"""

class MyConnectionPool(object):
    __pool = None
    def __init__(self,db_ip,db_port,db_name,db_user,db_password):
        self.db_ip = db_ip  # 数据库IP地址
        self.db_port = db_port  # 端口号
        self.db_name = db_name  # 数据库名
        self.db_user = db_user  # 用户名
        self.db_password = db_password  # 用户密码

        self.__pool = PooledDB(
            creator=pymysql,  # creator : 使用连接数据库的模块
            mincached=20,  # mincached : 启动时开启的闲置连接数量(缺省值 0 开始时不创建连接)
            maxcached=20,  # maxcached : 连接池中允许的闲置的最多连接数量(缺省值 0 代表不闲置连接池大小)
            maxshared=30,  # maxshared : 共享连接数允许的最大数量(缺省值 0 代表所有连接都是专用的)如果达到了最大数量,被请求为共享的连接将会被共享使用
            maxconnections=100,  # maxconnecyions : 创建连接池的最大数量(缺省值 0 代表不限制)
            blocking=True,  # blocking : 设置在连接池达到最大数量时的行为(缺省值 0 或 False 代表返回一个错误<toMany......> 其他代表阻塞直到连接数减少,连接被分配)
            maxusage=0,  # maxusage : 单个连接的最大允许复用次数(缺省值 0 或 False 代表不限制的复用).当达到最大数时,连接会自动重新连接(关闭和重新打开)
            setsession=None,  # setsession : 一个可选的SQL命令列表用于准备每个会话，如["set datestyle to german", ...]
            host=self.db_ip,  # 数据库IP地址
            port=self.db_port,  # 端口号
            user=self.db_user,  # 用户名
            passwd=self.db_password,  # 用户密码
            db=self.db_name,  # 数据库名
            use_unicode=True,
            charset="utf8"  # 数据库连接编码
        )

    # 创建数据库连接conn和游标cursor
    def __enter__(self):
        self.conn = self.__getconn()
        self.cursor = self.conn.cursor()

    # 创建数据库连接池
    def __getconn(self):
        if self.__pool is None:
            self.__pool = PooledDB(
                creator=pymysql,  # creator : 使用连接数据库的模块
                mincached=20,  # mincached : 启动时开启的闲置连接数量(缺省值 0 开始时不创建连接)
                maxcached=20,  # maxcached : 连接池中允许的闲置的最多连接数量(缺省值 0 代表不闲置连接池大小)
                maxshared=30,  # maxshared : 共享连接数允许的最大数量(缺省值 0 代表所有连接都是专用的)如果达到了最大数量,被请求为共享的连接将会被共享使用
                maxconnections=100,  # maxconnecyions : 创建连接池的最大数量(缺省值 0 代表不限制)
                blocking=True,  # blocking : 设置在连接池达到最大数量时的行为(缺省值 0 或 False 代表返回一个错误<toMany......> 其他代表阻塞直到连接数减少,连接被分配)
                maxusage=0,  # maxusage : 单个连接的最大允许复用次数(缺省值 0 或 False 代表不限制的复用).当达到最大数时,连接会自动重新连接(关闭和重新打开)
                setsession=None,  # setsession : 一个可选的SQL命令列表用于准备每个会话，如["set datestyle to german", ...]
                host=self.db_ip,  # 数据库IP地址
                port=self.db_port,  # 端口号
                user=self.db_user,  # 用户名
                passwd=self.db_password,  # 用户密码
                db=self.db_name,  # 数据库名
                use_unicode=True,
                charset="utf8"  # 数据库连接编码
            )
        return self.__pool.connection()

    # 释放连接池资源
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cursor.close()
        self.conn.close()

    # 从连接池中取出一个连接
    def getconn(self):
        conn = self.__getconn()
        cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
        cursor = conn.cursor(cursor=pymysql.cursors.DictCursor)
        return cursor, conn


"""执行语句查询有结果返回结果没有返回0；增/删/改返回变更数据条数，没有返回0"""


class MySqLHelper(object):
    def __init__(self, db_ip, db_port, db_name, db_user, db_password):
        self.db = MyConnectionPool(db_ip, db_port, db_name, db_user, db_password)  # 从数据池中获取连接

    # def __new__(cls, *args, **kwargs):
    #     if not hasattr(cls, 'inst'):  # 单例
    #         cls.inst = super(MySqLHelper, cls).__new__(cls, *args, **kwargs)
    #     return cls.inst

    # 封装执行命令
    def execute(self, sql, param=None, autoclose=False):
        """
        【主要判断是否有参数和是否执行完就释放连接】
        :param sql: 字符串类型，sql语句
        :param param: sql语句中要替换的参数"select %s from tab where id=%s" 其中的%s就是参数
        :param autoclose: 是否关闭连接
        :return: 返回连接conn和游标cursor
        """
        cursor, conn = self.db.getconn()  # 从连接池获取连接
        count = 0
        try:
            # count : 为改变的数据条数
            if param:
                count = cursor.execute(sql, param)
            else:
                count = cursor.execute(sql)
            conn.commit()
            if autoclose:
                self.close(cursor, conn)
        except Exception as e:
            print("db error_msg:", e.args)
        return cursor, conn, count

    # 释放连接
    def close(self, cursor, conn):
        """释放连接归还给连接池"""
        cursor.close()
        conn.close()

    # 查询所有
    def selectall(self, sql, param=None):
        try:
            cursor, conn, count = self.execute(sql, param)
            res = cursor.fetchall()
            return res
        except Exception as e:
            print("db error_msg:", e.args)
            self.close(cursor, conn)
            return count

    # 查询单条
    def selectone(self, sql, param=None):
        try:
            cursor, conn, count = self.execute(sql, param)
            res = cursor.fetchone()
            self.close(cursor, conn)
            return res
        except Exception as e:
            print("db error_msg:", e.args)
            self.close(cursor, conn)
            return count

    # 增加
    def insertone(self, sql, param):
        try:
            cursor, conn, count = self.execute(sql, param)
            conn.commit()
            self.close(cursor, conn)
            return count
        except Exception as e:
            print("db error_msg:", e.args)
            conn.rollback()
            self.close(cursor, conn)
            return count

    # 删除
    def delete(self, sql, param=None):
        try:
            cursor, conn, count = self.execute(sql, param)
            conn.commit()
            self.close(cursor, conn)
            return count
        except Exception as e:
            print("db error_msg:", e.args)
            conn.rollback()
            self.close(cursor, conn)
            return count

    # 修改
    def update(self, sql, param=None):
        try:
            cursor, conn, count = self.execute(sql, param)
            conn.commit()
            self.close(cursor, conn)
            return count
        except Exception as e:
            print("db error_msg:", e.args)
            conn.rollback()
            self.close(cursor, conn)
            return count


if __name__ == '__main__':
    db = MySqLHelper()
    while True:
        sql1 = 'select * from space where unit=%s'
        args = '11'
        ret = db.selectone(sql=sql1, param=args)
        print(ret)  # (79, 2, 1, 2.2, None, 2, 1)
    # 查询单条
    sql1 = 'select * from space where unit=%s'
    args = '11'
    ret = db.selectone(sql=sql1, param=args)
    print(ret)  # (79, 2, 1, 2.2, None, 2, 1)
    # 查询所有
    sql2 = "select * from space"
    ret = db.selectall(sql=sql2)
    print(ret)
    # 增加
    sql3 = 'insert into vehicle (car_number,primary_key) VALUES (%s,%s)'
    ret = db.insertone(sql3, ('鄂A6X3B0', 'DASDASDEFDFSDASDADASDAS'))
    print(ret)
    # 删除
    sql4 = 'delete from vehicle WHERE car_number=%s'
    args = '鄂A6X3B0'
    ret = db.delete(sql4, args)
    print(ret)
    # 修改
    sql5 = 'update command_queue set export_id=%s WHERE car_number LIKE %s'
    args = ('100', 'WK0001')
    ret = db.update(sql5, args)
    print(ret)
