import json
import time
from functools import partial

import requests
from PyQt5.QtCore import QSize
from PyQt5.QtCore import pyqtSignal, QTimer, Qt
from PyQt5.QtGui import QFont, QIcon, QPixmap, QCursor
from PyQt5.QtWidgets import QMessageBox, QListWidgetItem, QMainWindow, QMenu, QPushButton, QGridLayout

import google.protobuf.text_format as tf

import 管理节点.ui.spaceUi as sui
from 管理节点.window_controller.pickup_window import PickupWindow, ResultReturn
import 管理节点.message.message_pb2 as message


class MainWindow(QMainWindow, sui.Ui_MainWindow):
    # # 取车界面信号
    # signal_dict = pyqtSignal(dict)

    def __init__(self, parking_config, g_space, g_rabbitmq, parent=None):
        super(MainWindow, self).__init__(parent)

        # 配置导入
        self.parking_config = parking_config
        self.g_space = g_space
        self.g_rabbitmq = g_rabbitmq

        self.setupUi(self)
        self.setWindowTitle(parking_config['window_title'])
        font = QFont()
        font.setFamily('宋体')
        font.setBold(False)
        font.setPointSize(15)
        self.parking_listWidget.setFont(font)
        self.pushButton_grade1.setIcon(QIcon(QPixmap("./images/grade1.png")))
        self.pushButton_grade2.setIcon(QIcon(QPixmap("./images/grade2.png")))
        self.pushButton_grade3.setIcon(QIcon(QPixmap("./images/grade3.png")))
        self.pushButton_grade1.setText("限高:" + str(parking_config['grade1']))
        self.pushButton_grade2.setText("限高:" + str(parking_config['grade2']))
        self.pushButton_grade3.setText("限高:" + str(parking_config['grade3']))

        if parking_config['unit'] == 1:
            self.unit_label.setText("A")
        elif parking_config['unit'] == 2:
            self.unit_label.setText("B")
        elif parking_config['unit'] == 11:
            self.unit_label.setText("A-1")
        elif parking_config['unit'] == 12:
            self.unit_label.setText("A-2")
        elif parking_config['unit'] == 13:
            self.unit_label.setText("A-3")
        elif parking_config['unit'] == 14:
            self.unit_label.setText("A-4")
        elif parking_config['unit'] == 21:
            self.unit_label.setText("B-1")
        elif parking_config['unit'] == 22:
            self.unit_label.setText("B-2")
        elif parking_config['unit'] == 23:
            self.unit_label.setText("B-3")
        elif parking_config['unit'] == 24:
            self.unit_label.setText("B-4")
        elif parking_config['unit'] == 25:
            self.unit_label.setText("B-5")
        elif parking_config['unit'] == 31:
            self.unit_label.setText("C-1")
        elif parking_config['unit'] == 32:
            self.unit_label.setText("C-2")
        self.btn_positions = [(i, j) for i in range(parking_config['row'], -1, -1) for j in
                              range(parking_config['column'])]
        self.space_dict = {}
        self.space_is_init = False
        self.show_model = self.car_number_rbtn_2.isChecked()

        self.rmq_statu_dict = {}
        self.command_dict = {}

        self.timer = QTimer()
        self.timer.timeout.connect(self.Switch)
        self.timer.start(200)

        self.find_btn.clicked.connect(self.find_btn_click)

        # 绑定停车数据显示控件
        self.parking_times.setText("停车次数")
        self.car_quantity.setText("停车车数")
        self.data_by_day_radioButton.toggled.connect(self.show_parkingData)
        self.data_by_month_radioButton.toggled.connect(self.show_parkingData)

        # # 取车窗口.默认隐藏
        # self.pickup_window = PickupWindow()

        # self.pickupWindow.hide()
        # 绑定取车界面槽函数.传输车位信息(主窗口->取车窗口)
        # self.signal_dict.connect(self.pickupWindow.get_data_dict)

    # 刷新停车数据
    def show_parkingData(self):
        date = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        # date = "2022-11-16 15:22:07" # date = "2022-12-11 15:22:07"

        if self.data_by_day_radioButton.isChecked():
            value_parking_times = self.g_space.query_parkingTime_inPeriod("day", date)
            value_car_quantity = self.g_space.query_carQuantity_inPeriod("day", date)
            text1 = "停车{value}次".format(value=value_parking_times)
            text2 = "停车{value}辆".format(value=value_car_quantity)
            self.parking_times.setText(text1)
            self.car_quantity.setText(text2)
        elif self.data_by_month_radioButton.isChecked():
            value_parking_times = self.g_space.query_parkingTime_inPeriod("month", date)
            value_car_quantity = self.g_space.query_carQuantity_inPeriod("month", date)
            text1 = "停车{value}次".format(value=value_parking_times)
            text2 = "停车{value}辆".format(value=value_car_quantity)
            self.parking_times.setText(text1)
            self.car_quantity.setText(text2)

    def getBackGroundColor(self, color):
        if color == "red":
            return "background-color:rgb(255,70,70)"
        elif color == "gray":
            return "background-color:rgb(168,168,168)"
        elif color == "green":
            return "background-color:rgb(0,180,0)"
        elif color == "yellow":
            return "background-color:rgb(248,239,71)"
        elif color == "pastel":
            return "background-color:rgb(241,158,194)"

    def get_name(self, key):
        if key == 'id':
            return '下标'
        elif key == 'table_id':
            return '表编号'
        elif key == 'floor':
            return '楼层'
        elif key == 'height':
            return '限高'
        elif key == 'car_number':
            return '车牌号'
        elif key == 'statu':
            return '车位状态'
        elif key == 'plate_color':
            return '号牌颜色'
        elif key == 'plate_type':
            return '号牌类型'
        elif key == 'plate_confidence':
            return '号牌可信度'
        elif key == 'recognition_time':
            return '识别时间'
        elif key == 'primary_key':
            return '唯一码'
        else:
            return '未知'

    # 反向获取数据库字段名
    def get_tableName(self, key):
        if key == '下标':
            return 'id'
        elif key == '表编号':
            return 'table_id'
        elif key == '楼层':
            return 'floor'
        elif key == '限高':
            return 'height'
        elif key == '车牌号':
            return 'car_number'
        elif key == '车位状态':
            return 'statu'
        elif key == '号牌颜色':
            return 'plate_color'
        elif key == '号牌类型':
            return 'plate_type'
        elif key == '号牌可信度':
            return 'plate_confidence'
        elif key == '识别时间':
            return 'recognition_time'
        elif key == '唯一码':
            return 'primary_key'
        else:
            return '未知'

    def show_parking_details(self, park_dict):
        self.parking_listWidget.clear()
        for poit in park_dict.items():
            item = QListWidgetItem()
            item.setSizeHint(QSize(50, 42))
            name = self.get_name(poit[0])
            if name == '未知':
                continue
            value = poit[1]
            if name == '楼层':
                value = value - self.parking_config['floor_minus']
            item.setText(name + "：" + str(value))
            self.parking_listWidget.addItem(item)
        res = self.g_space.query_space_in_car_number(park_dict['car_number'])
        self.plate_full_image_label.setPixmap(QPixmap(""))
        self.plate_clip_image_label.setPixmap(QPixmap(""))
        if len(res) > 0:
            try:
                rep = requests.get(res[0]['plate_full_image'])
                pix = QPixmap()
                pix.loadFromData(rep.content)
                pix = pix.scaled(self.plate_full_image_label.size(), Qt.KeepAspectRatio)
                self.plate_full_image_label.setPixmap(pix)
            except Exception as e:
                print('全景图：' + str(e.args))
            try:
                rep = requests.get(res[0]['plate_clip_image'])
                pix = QPixmap()
                pix.loadFromData(rep.content)
                pix = pix.scaled(self.plate_clip_image_label.size(), Qt.KeepAspectRatio)
                self.plate_clip_image_label.setPixmap(pix)
            except Exception as e:
                print('特写图：' + str(e.args))

    def find_btn_click(self):
        text = self.find_lineEdit.text()
        if text == '':
            QMessageBox.question(None, "提示", "请输入要查询的内容",
                                 QMessageBox.Ok)
            return
        if self.car_number_rbtn.isChecked():
            park_dict = self.g_space.query_space_in_car_number(text)
            if len(park_dict) > 0:
                self.show_parking_details(park_dict[0])
            else:
                QMessageBox.question(None, "查询结果", "车牌号:" + text + "  未查询到数据！",
                                     QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容
        elif self.key_rbtn.isChecked():
            park_dict = self.g_space.query_space_in_primary_key(text)
            if len(park_dict) > 0:
                self.show_parking_details(park_dict[0])
            else:
                QMessageBox.question(None, "查询结果", "唯一码:" + text + "  未查询到数据！",
                                     QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容
        elif self.parking_rbtn.isChecked():
            park_dict = self.g_space.query_space_in_unit_and_table_id(self.parking_config['unit'], text)
            if len(park_dict) > 0:
                self.show_parking_details(park_dict[0])

            else:
                QMessageBox.question(None, "查询结果", "车位id:" + text + "  未查询到数据！",
                                     QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容

        return

    def park_btn_left_click(self):
        sender = self.sender()
        tool_tip = sender.toolTip()
        dict = json.loads(tool_tip)
        self.show_parking_details(dict)

    def park_btn_right_click(self):
        menu = QMenu(self)
        sender = self.sender()
        tool_tip = sender.toolTip()
        dict = json.loads(tool_tip)
        action = menu.addAction('启用车位')
        action.triggered.connect(partial(self.btn_opeSpace, dict, 0))
        action = menu.addAction('停用车位')
        action.triggered.connect(partial(self.btn_disableSpace, dict, 10))
        action = menu.addAction('清除车位')
        action.triggered.connect(partial(self.btn_clearSpace, dict))
        action = menu.addAction('取出车辆')
        action.triggered.connect(partial(self.btn_pickUp, dict))
        menu.exec_(QCursor.pos())

    def btn_disableSpace(self, dict, statu):
        res = self.g_space.update_space_status(dict["id"], statu)
        if res == 0:
            QMessageBox.question(None, '提示', '更新车位失败！请检查是否点击错误！',
                                 QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容

    def btn_opeSpace(self, dict, statu):
        res = self.g_space.update_space_status(dict["id"], statu)
        if res == 0:
            QMessageBox.question(None, '提示', '更新车位失败！请检查是否点击错误！',
                                 QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容

    def btn_clearSpace(self, dict):
        res = self.g_space.clear_space_data(dict["id"])
        if res == 0:
            QMessageBox.question(None, '提示', '更新车位失败！请检查是否点击错误！',
                                 QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容

    def btn_pickUp(self, dict):
        # # 使用信号传输车位信息
        # self.signal_dict.emit(dict)

        # 输出顺序
        number_list = ("车牌号", "车位id", "楼层")
        # 输出内容
        mapping_list = (dict[self.get_tableName("车牌号")],
                        dict[self.get_tableName("下标")],
                        int(dict[self.get_tableName("楼层")]) - int(self.parking_config['floor_minus'])
                        )
        # 组装显示信息
        info = []
        for i in range(len(number_list)):
            info.append(number_list[i] + ":" + str(mapping_list[i]))
        # 实例取车窗口,并以模态窗口形式显示
        pickup_window = PickupWindow(info)
        pickup_window.exec_()

        res = pickup_window.getResult()
        if res is ResultReturn.OK:
            table = message.pick_table()
            table.primary_key = dict["primary_key"]
            self.g_rabbitmq.publish("command_ex", "user_pick_command_request_port",
                                    tf.MessageToString(table, as_utf8=True))
            QMessageBox.question(None, '提示', '取车消息发送成功！',
                                 QMessageBox.Ok)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容

    def get_btn_show(self, park):
        font = QFont()
        font.setFamily('宋体')
        font.setPointSize(21)
        if park["statu"] == 6:
            text_str = "入   口"
            font.setBold(True)
        elif park["statu"] == 7:
            text_str = "出   口"
            font.setBold(True)
        elif park["statu"] == 8:
            text_str = " 控制室 "
            font.setBold(True)
        elif park["statu"] == 2 and park["car_number"] is not None and self.car_number_rbtn_2.isChecked():
            text_str = park["car_number"]
            font.setPointSize(20)
        else:
            text_str = "车位:%d\n楼层:%d" % (park["table_id"], (park["floor"] - self.parking_config['floor_minus']))
            font.setPointSize(10)

        background_color = ""
        background_image = ""

        if park["statu"] == 10:
            background_color = "background-color:rgb(255,130,130);"
        elif park["statu"] == 2 and park["car_number"] is not None:
            background_color = "background-color:rgb(248,239,71);"
        elif park["statu"] == 6 or park["statu"] == 7 or park["statu"] == 8:
            background_color = "background-color:rgb(0,162,232);"
        elif park["statu"] == 0:
            background_color = "background-color:rgb(195,195,195);"
        # 设置背景图
        if park["height"] == self.parking_config['grade1']:
            background_image = "background-image: url(./images/grade1.png);"
        elif park["height"] == self.parking_config['grade2']:
            background_image = "background-image: url(./images/grade2.png);"
        elif park["height"] == self.parking_config['grade3']:
            background_image = "background-image: url(./images/grade3.png);"
        stylesheet = background_color + "pressed{padding-top:2px;padding-left:2px;};" + background_image + "background-repeat:no-repeat;"

        tool_tip = '{"id":%d,\n"table_id":%d,\n"floor":%d, \n"height":%f,\n"car_number":"%s" , \n"statu":%d,\n"plate_color":"%s" ,\n"plate_type":"%s" ,\n"plate_confidence":"%s",\n"recognition_time":"%s",\n"primary_key":"%s"}' % (
            park["id"], (-1 if park["table_id"] is None else park["table_id"]),
            park["floor"], (0 if park["height"] is None else park["height"]),
            park["car_number"], park["statu"], park["plate_color"],
            park["plate_type"], park["plate_confidence"], park["recognition_time"], park["primary_key"])
        return font, text_str, stylesheet, tool_tip

    def Switch(self):

        # 绘制车位按钮
        space_dict = self.g_space.query_space_in_unit(self.parking_config['unit'])
        if len(space_dict) < 0:
            return
        is_show_car_number = self.car_number_rbtn_2.isChecked()
        if self.space_dict != space_dict:
            if self.space_is_init == False:
                self.space_dict = space_dict
                self.space_is_init = True
                is_drawIn = False
                is_drawOut = False
                is_drawControlRoom = False

                for park, position in zip(self.space_dict, self.btn_positions):
                    btn = QPushButton()
                    font, text_str, stylesheet, tool_tip = self.get_btn_show(park)
                    btn.setFont(font)
                    btn.setLayout(QGridLayout())
                    btn.setText(text_str)
                    btn.setStyleSheet(stylesheet)
                    btn.setToolTip(tool_tip)
                    btn.setObjectName(str(park["table_id"]))
                    if text_str == "入   口" and is_drawIn is False:
                        self.unit_gridLayout.addWidget(btn, *position, 1, self.parking_config['in_occupy_space'])
                        is_drawIn = True
                    elif text_str == "出   口" and is_drawOut is False:
                        self.unit_gridLayout.addWidget(btn, *position, 1, self.parking_config['out_occupy_space'])
                        is_drawOut = True
                    elif text_str == " 控制室 " and is_drawControlRoom is False:
                        self.unit_gridLayout.addWidget(btn, *position, 1,
                                                       self.parking_config['controlroom_occupy_space'])
                        is_drawControlRoom = True
                    elif text_str != "出   口" and text_str != "入   口" and text_str != " 控制室 ":
                        btn.setContextMenuPolicy(Qt.CustomContextMenu)
                        btn.customContextMenuRequested.connect(self.park_btn_right_click)
                        btn.clicked.connect(self.park_btn_left_click)
                        self.unit_gridLayout.addWidget(btn, *position)
            else:
                for park1, park2 in zip(self.space_dict, space_dict):
                    if park1 != park2:
                        font, text_str, stylesheet, tool_tip = self.get_btn_show(park2)
                        results = self.findChild(QPushButton, str(park2["table_id"]))
                        results.setFont(font)
                        results.setText(text_str)
                        results.setStyleSheet(stylesheet)
                        results.setToolTip(tool_tip)

                self.space_dict = space_dict
        if self.show_model != is_show_car_number:
            self.show_model = is_show_car_number
            for park in self.space_dict:
                if park["statu"] == 2:
                    font, text_str, stylesheet, tool_tip = self.get_btn_show(park)
                    results = self.findChild(QPushButton, str(park["table_id"]))
                    results.setFont(font)
                    results.setText(text_str)
                    results.setStyleSheet(stylesheet)
                    results.setToolTip(tool_tip)

    def closeEvent(self, event):
        results = QMessageBox.question(self, '退出', '你确定要退出吗?', QMessageBox.Yes | QMessageBox.No,
                                       QMessageBox.No)  # "退出"代表的是弹出框的标题,"你确认退出.."表示弹出框的内容
        if results == QMessageBox.Yes:
            self.g_rabbitmq.close()
            event.accept()  # 接受关闭事件

        else:
            event.ignore()  # 忽略关闭事件
